// Variable that holds the data about queries and their results
var data;

// Variable that holds the number of the query that is currently displayed (not necessarily selected in the dropdown)
var query;

// Variable that specifies which results should be picked
var results;

// Variable that specifies which topic should be picked
var topic;

/**
 * Reads all data needed for the search engine.
 */
function loadData() {
  var urlParams = new URLSearchParams(window.location.search);
  if (window.atob(urlParams.get("t")) == "v") {
    topic = "vegetarian";
  } else if (window.atob(urlParams.get("t")) == "s") {
    topic = "schooluniforms";
  }

  results = "ranking_".concat(
    window.atob(urlParams.get("r")),
    "_2"
  );

  query = window.atob(urlParams.get("q"));

  // Temporarily turn off asynchronous ajax calls
  jQuery.ajaxSetup({ async: false });

  // Read the query/results file
  $.getJSON("./json/".concat(topic, ".json"), function (json) {
    data = json;

    // Retrieve each result
    for (var i = 0; i < json.length; i++) {
      for (var j = 0; j < json[i][results].length; j++) {
        var filename = json[i][results][j];

        $.get(
          "./documents/".concat(topic, "/", filename),
          function (text) {
            var splitText = text.split("\n");

            // Process the result into a conventional format
            var result = {};
            result["title"] = splitText[0];
            result["content"] = splitText.slice(1).join("\n");
            result["all"] = text;

            data[i][results][j] = result;
          },
          "text"
        );
      }
    }
  });

  // Turn asynchronous ajax calls back on
  jQuery.ajaxSetup({ async: true });
}

/**
 * Change the results for the specified query.
 *
 * @param {*} number index of the query in the select box.
 */
function setQuery() {
  // Change all the results
  for (var i = 0; i < data[query][results].length; i++) {
    var title = document.getElementById("title-".concat(i));
    var preview = document.getElementById("preview-".concat(i));

    var result = data[query][results][i];

    title.innerHTML = marked(result["title"]);
    title.firstChild.setAttribute("rank", i + 1);
    preview.innerHTML = result["content"];
  }

  // Go to the first page of results
  setPage(0);
}

/**
 * Changes the results that are shown on the screen.
 * @param {*} page index of page of results that needs to be shown.
 */
function setPage(page) {
  // Temporarily hide the results page
  var resultspage = document.getElementById("searchresults");
  resultspage.style.display = "none";

  // Temporarily hide the pagination
  var pagination = document.getElementById("pagination");
  pagination.style.display = "none";

  // Show the spinner
  var spinner = document.getElementById("spinner");
  spinner.style.display = "block";

  // Set the visibility of the first results page
  var firstpage = document.getElementsByClassName("firstpage");
  for (var i = 0; i < firstpage.length; i++) {
    firstpage[i].style.display = page == 0 ? "block" : "none";
  }

  // Set the visibility of the second page
  var secondpage = document.getElementsByClassName("secondpage");
  for (var i = 0; i < firstpage.length; i++) {
    secondpage[i].style.display = page == 1 ? "block" : "none";
  }

  // Change the pagination style
  var pagination = document.getElementsByClassName("page-item");
  for (var i = 0; i < pagination.length; i++) {
    pagination[i].classList.remove("active");
  }

  pagination[page].classList.add("active");

  setTimeout(function () {
    // Hide the spinner
    var spinner = document.getElementById("spinner");
    spinner.style.display = "none";

    // Show the results page
    var resultspage = document.getElementById("searchresults");
    resultspage.style.display = "block";

    // Show the pagination
    var pagination = document.getElementById("pagination");
    pagination.style.display = "block";
  }, 500 + Math.random() * 500);
}

/**
 * Opens a result in an overlay.
 * @param {*} result number of the result that is clicked.
 */
function clickResult(result) {
  // Disable the elements in the background
  var children = document.getElementById("results").getElementsByTagName("*");
  for (var i = 0; i < children.length; i++) {
    children[i].style.pointerEvents = "none";
  }

  // Disable scrolling of the body
  var body = document.body;
  body.style.overflow = "hidden";

  // Change the content shown in the result overlay
  var resultContent = document.getElementById("resultContent");
  resultContent.innerHTML = marked(data[query][results][result]["all"]);

  // Show the result overlay
  var resultOverlay = document.getElementById("resultOverlay");
  resultOverlay.style.display = "block";

  // Always start at the beginning of the result
  resultOverlay.scrollTop = 0;
}

/**
 * Closes the result overlay.
 */
function closeResult() {
  // Enable the elements in the background
  var children = document.getElementById("results").getElementsByTagName("*");
  for (var i = 0; i < children.length; i++) {
    children[i].style.pointerEvents = "auto";
  }

  // Enable scrolling of the body
  var body = document.body;
  body.style.overflow = "auto";

  // Hide the result overlay
  resultOverlay.style.display = "none";
}

/**
 * Changes the results shown based on the selected query in the select box.
 */
function search() {
  setQuery();
}

// When the page is loaded, load the data
window.onload = function () {
  loadData();

  search();

  // LogUI configuration object
  var urlParams = new URLSearchParams(window.location.search);
  let configurationObject = {
    logUIConfiguration: {
      endpoint: "YOUR_ENDPOINT",
      authorisationToken: "YOUR_TOKEN",
      verbose: false,
      browserEvents: {
        contextMenu: false,
        URLChanges: false,
        pageFocus: true,
        trackCursor: true,
        cursorUpdateFrequency: 100,
        pageResize: false,
      },
    },
    applicationSpecificData: {
      userID: urlParams.get("PROLIFIC_PID"),
      topic: topic,
      q: window.atob(urlParams.get("q")),
      ranking: window.atob(urlParams.get("r")),
    },
    trackingConfiguration: {
      "close-click": {
        selector: "#span-close",
        event: "click",
        name: "CLICK_CLOSE",
      },
      "page-1-click": {
        selector: "#a-page-1",
        event: "click",
        name: "CLICK-PAGE-1",
      },
      "page-2-click": {
        selector: "#a-page-2",
        event: "click",
        name: "CLICK-PAGE-2",
      },
      "all-click": {
        selector: "*",
        event: "click",
        name: "CLICK_ANY",
      },
      "result-click": {
        selector: ".title *",
        event: "click",
        name: "CLICK_RESULT",
        metadata: [
          {
            nameForLog: "resultRank",
            sourcer: "elementAttribute",
            lookFor: "rank",
          },
        ],
      },
      "result-mousemovements": {
        selector: ".searchresult",
        event: "mouseHover",
        properties: {
          mouseenter: {
            name: "RESULT_HOVER_IN",
          },
          mouseleave: {
            name: "RESULT_HOVER_OUT",
          },
        },
        metadata: [
          {
            nameForLog: "resultRank",
            sourcer: "elementAttribute",
            lookFor: "rank",
          },
        ],
      },
    },
  };

  LogUI.init(configurationObject);
};

window.onbeforeunload = function() {
  LogUI.stop();
}
